# chiller_exporter - Rittal Chiller metrics exporter for Prometheus

`chiller_exporter` is a metric exporting tool written in python.

It reads metrics from one or more chiller via http request and expose them over HTTP,
 to be read by the Prometheus server.

## Getting Started

These instructions will get you a copy of the project up and running on your 
 local machine for development and testing purposes. See deployment for notes
 on how to deploy the project on a live system.

### Prerequisites

* Python 3.5 or newer

Create a Python 3 virtual environment:
```
$ python3 -m venv chiller_exporter
$ source chiller_exporter/bin/activate
```

### Installing

Install `chiller_exporter` with pip:

```
$ python3 -m pip install git+ssh://git@git.mgmt.innovo-cloud.de/innovo-cloud/prometheus-chiller-exporter.git
$ OR, to checkout at commit 953f8a13
$ python3 -m pip install git+ssh://git@git.mgmt.innovo-cloud.de/innovo-cloud/prometheus-chiller-exporter.git@953f8a13
```

Available options can be displayed with ```chiller_exporter -h```:

```
usage: chiller_exporter [-h] [-a app] [-c config] [-m metric] [-p port]
                        [-t timeout]
                        [chillers [chillers ...]]

chiller exporter args

positional arguments:
  chillers              rittal chiller urls as "instance=url ..."

optional arguments:
  -h, --help            show this help message and exit
  -a app, --app app     prometheus application prefix
  -c config, --config config
                        config file
  -m metric, --metric-conf metric
                        config file for pcoweb metrics
  -p port, --port port  Listen to this port
  -t timeout, --timeout timeout
                        chiller http timeout

```

### Quick start

Define your chiller(s) with ip or fqdn in a file `chiller.conf`
```ini
[chiller_0]
  hostname = 10.104.2.1
[chiller_1]
  hostname = 10.104.2.2
...
```
and run ```chiller_exporter -c chiller.conf```
 
## Configuration File Format

The file format used by `chiller_exporter` is similar to the format used by older versions of Microsoft Windows. It
 consists of one or more named sections, each of which can contain individual options with names and values.

Config file sections are identified by looking for lines starting with [ and ending with ]. The value between the square
 brackets is the section name, and can contain any characters except square brackets.

Options are listed one per line within a section. The line starts with the name of the option, which is separated from
 the value by a colon (:) or equal sign (=). Whitespace around the separator is ignored when the file is parsed. 

Lines starting with semi-colon (;) or octothorpe (#) are treated as comments and not visible when accessing the contents
 of the configuration file programmatically.

### Chiller configuration

The section name is used as value for the metric label "instance". It can be any sequence of (even UTF-8)
 characters, but the backslash, the double-quote, and the line-feed characters have to be escaped as \, ", and \n,
 respectively.

`chiller_exporter` knows the following option names:
  
 - scheme (default: http)
 - hostname
 - port (default: 80)
 - path (default: /usr-cgi/xml.cgi)
 - query (default: N%7CD%7C1%7C207%7CI%7C1%7C207%7CA%7C1%7C207)
 - timeout (default: 2.0)
 
Since all options except hostname have reasonable default values, they can be omitted. 

### Metric configuration

The currently known chiller metrics exposed by the http service as xml are defined in `pcoweb.conf` located in the
 source directory of `chiller_exporter`. Feel free to make a copy and change or add metric definitions. Your version of
 metric configuration can be specified by using the command line parameter -m (or --metric)
 
Three groups of chiller metrics are known and used as section names: DIGITAL, INTEGER and ANALOG. The options in these 
 sections have the format ```metric_name = chiller_id|chiller_help```

metric_name is used as prometheus metric name, prefixed by command line option -a (--app, default 'chiller'). The 
 combination of section name and chiller_id is used to reference the metric value in the xml output of the chiller.
 chiller_help is used as help text in the prometheus exposition format. chiller_id and chiller_help is separated by a
 pipe symbol. chiller_help may contains any character (even the pipe symbol) until end of line (special characters have
 to be escaped).


### Example

For a configured chiller "chiller_0" in chiller.conf
```
[chiller_0]
  hostname = 10.104.2.1
```

 and a metric.conf 
```
[ANALOG]
  setpoint_medium = 61|Setpoint Medium|°C
```

and a given xml from chiller
```
<PCOWEB t="2017-11-10 11:00 ">
<PCO>
<ANALOG>
<O I="61" V="16.0"/>
</ANALOG>
</PCO>
</PCOWEB>
```

the exporters output to prometheus is:
```
# HELP chiller_setpoint_medium Setpoint Medium|°C
# TYPE chiller_setpoint_medium gauge
chiller_setpoint_medium{instance="chiller_0"} 16.0
```

## Development and Testing with Docker

The project includes a docker-compose.yml and a Dockerfile, which uses the chiller.conf-sample as chiller.conf with
 10.104.2.22 as "chiller_0":
 
Run ```docker-compose build``` followed by ```docker-compose up``` and open
 [http://localhost:9118](http://localhost:9118) (exporter) and 
 [http://localhost:9090](http://localhost:9090) (prometheus) in a web browser.