import time

from prometheus_client import start_http_server, REGISTRY

from .config import parse_args, parse_config, parse_metrics_config
from .chiller import ChillerCollector


def main():
    """Parse command line, load metric and chiller configuration, and configure logging"""
    try:
        args = parse_args()

        metrics = parse_metrics_config(args.metric_conf, args.app)

        chillers = parse_config(args)

        REGISTRY.register(ChillerCollector(args.app, chillers, metrics))
        start_http_server(int(args.port))

        print('Polling')
        for label, url in chillers.items():
            print('  {} as {}'.format(url, label))
        print('Serving at port: {}'.format(int(args.port)))
        while True:
            time.sleep(1)

    except KeyboardInterrupt:
        print(' Interrupted')
        exit(0)


if __name__ == "__main__":
    main()
